function varargout = dynare_gui(varargin)
% function varargout = dynare_gui(varargin)
% Runs DYNARE_GUI.
%      DYNARE_GUI, by itself, creates a new DYNARE_GUI or raises the existing
%      singleton*.
%
%      H = DYNARE_GUI returns the handle to a new DYNARE_GUI or the handle to
%      the existing singleton*.interface for the DYNARE estimation command
%
%      DYNARE_GUI('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in DYNARE_GUI.M with the given input arguments.
%
%      DYNARE_GUI('Property','Value',...) creates a new DYNARE_GUI or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before dynare_gui_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to dynare_gui_OpeningFcn via varargin.
%
%
% Copyright (C) 2003-2019 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  1, ...
                   'gui_OpeningFcn', @dynare_gui_OpeningFcn, ...
                   'gui_OutputFcn',  @dynare_gui_OutputFcn, ...
                   'gui_LayoutFcn',  @dynare_gui_LayoutFcn, ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

end

% --- Executes just before dynare_gui is made visible.
function dynare_gui_OpeningFcn(hObject, ~, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to dynare_gui (see VARARGIN)

% Choose default command line output for dynare_gui
handles.output = hObject;
movegui(hObject,'center');

global dynareroot

setappdata(0, 'bg_color', 'default');
setappdata(0, 'special_color', 'white');
setappdata(0, 'main_figure', hObject);

warning_config();
dynareroot = dynare_config('', false);

setappdata(0, 'dynare_gui_root', [fileparts(which('dynare_gui')) filesep]);

% Update handles structure
guidata(hObject, handles);
end

% --- Outputs from this function are returned to the command line.
function varargout = dynare_gui_OutputFcn(~, ~, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;
end

% --------------------------------------------------------------------
function project_new_Callback(hObject, ~, ~)

global M_ oo_ options_
global project_info model_settings

%% Reset GUI data
if ~isempty(project_info)
    % Close existing project
    if isstruct(project_info) ...
            && ~isempty(fieldnames(project_info)) ...
            && isfield(project_info, 'project_name') ...
            && ~isempty(project_info.project_name)
        try
            gui_close_project();
        catch
        end
    end
else
    if ~isempty(model_settings)
        evalin('global', 'clear model_settings')
    end
end

gui_set_project_info()

%% Handle Dynare Data
if (exist('M_', 'var') && isstruct(M_)) || (exist('oo_', 'var') && isstruct(oo_))
    answer = questdlg(...
        'It seems like you have already run dynare. Do you want to use the output in the workspace?', ...
        'DynareGUI', 'Yes', 'No', 'Yes');
    if strcmp(answer, 'No')
        evalin('global', 'clear M_ options_ oo_ estim_params_ bayestopt_ dataset_ dataset_info estimation_info ys0_ ex0_');
    else
        if ~(exist([M_.fname filesep 'model' filesep 'json' filesep 'modfile-original.json'], 'file') == 2 ...
                && exist([M_.fname filesep 'model' filesep 'json' filesep 'modfile.json'], 'file') == 2)
            gui_tools.show_error('The .mod file was not run with the json=compute option. Please run it again.');
            return
        end
        project_info.mod_file = M_.fname;
        project_info.mod_file_already_run = true;
        if isfield(options_, 'datafile') && ischar(options_.datafile) && ~isempty(options_.datafile)
            % TODO: set dates correctly based on info in options_
            project_info.observed_variables_set = true;
        end
        if project_info.project_model_stochastic
            if isfield(oo_, 'dr') && isfield(oo_, 'mean') && isfield(oo_, 'var') && isfield(oo_, 'autocorr')
                project_info.stochastic_simulation_run = true;
            end
            if isfield(oo_, 'datafile') || isfield(oo_, 'dataset')
                project_info.observed_variables_set = true;
            end
            if isfield(oo_, 'SmoothedVariables')
                project_info.calibrated_smoother_run = true;
            end
            if isfield(oo_, 'dr') && isfield(oo_.dr, 'ghu')
                project_info.estimation_run = true;
            end
            if isfield(oo_, 'shock_decomposition') && ~isempty(oo_.shock_decomposition)
                project_info.shock_decomposition_run = true;
            end
            if isfield(oo_, 'realtime_shock_decomposition') && ~isempty(oo_.realtime_shock_decomposition)
                project_info.realtime_shock_decomposition_run = true;
            end
        else
            if isfield(oo_, 'endo_simul') || isfield(oo_, 'exo_simul')
                project_info.deterministic_simulation_run = true;
            end
        end
    end
end

gui_project(addTab(hObject, 'New project'), 'New');
end

% --------------------------------------------------------------------
function project_open_Callback(varargin)
% hObject    handle to project_open (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global project_info

% Close existing project
if isstruct(project_info) ...
        && ~isempty(fieldnames(project_info)) ...
        && isfield(project_info, 'project_name') ...
        && ~isempty(project_info.project_name)
    try
        gui_close_project();
    catch
    end
end

gui_open_project(varargin{1});
end

% --------------------------------------------------------------------
function project_close_Callback(~, ~, ~)
gui_close_project();
end

% --------------------------------------------------------------------
function project_save_Callback(hObject, ~, ~)
gui_save_project(hObject, 'Save');
end

% --------------------------------------------------------------------
function project_save_as_Callback(hObject, ~, ~)
gui_save_project(hObject, 'Save As');
end

% --------------------------------------------------------------------
function project_exit_Callback(~, ~, ~)
global project_info
% Before quitting check if current project has been modified and ask 'Save changes to current project?'
if isstruct(project_info) ...
        && isfield(project_info, 'project_name') ...
        && ~isempty(project_info.project_name) ...
        && isfield(project_info, 'modified') ...
        && project_info.modified
    answer = questdlg(sprintf('Do you want to save changes to project %s?', project_info.project_name),'DynareGUI','Yes','No','Cancel','Yes');
    if strcmp(answer,'Yes')
        gui_tools.save_project();
    elseif strcmp(answer,'Cancel')
        return
    end    
end

%evalin('base','diary off;');
appdata = getappdata(0);
fns = fieldnames(appdata);
for ii = 1:numel(fns)
    rmappdata(0,fns{ii});
end
evalin('base','clear');
close force
end


% Model!
% --------------------------------------------------------------------
function model_load_Callback(hObject, ~, ~)
gui_load_mod_file(hObject);
end

% --------------------------------------------------------------------
function model_settings_Callback(hObject, ~, ~)
possibly_create_model_settings();
gui_define_model_settings(hObject);
end

% --------------------------------------------------------------------
function model_save_snapshot_Callback(~, ~, ~)
gui_tools.save_model_snapshot();
end

% --------------------------------------------------------------------
function model_load_snapshot_Callback(~, ~, ~)
gui_tools.load_model_snapshot();
end

% --------------------------------------------------------------------
function model_export_Callback(~, ~, ~)
gui_tools.gui_export_to_mod_file();
end

% --------------------------------------------------------------------
function model_logfile_Callback(hObject, ~, ~)
gui_log_file(hObject);
end

% Estimation!
% --------------------------------------------------------------------
function estimation_observed_variables_Callback(hObject, ~, ~)
possibly_create_model_settings();
gui_observed_vars(addTab(hObject, 'Observed vars. '));
end

% --------------------------------------------------------------------
function estimation_parameters_shocks_Callback(hObject, ~, ~)
gui_estim_params(addTab(hObject, 'Est. params & shocks '));
end

% --------------------------------------------------------------------
function estimation_run_Callback(hObject, ~, ~)
possibly_create_model_settings();
gui_estimation(addTab(hObject, 'Estimation '));
end

% --------------------------------------------------------------------
function estimation_run_calibrated_smoother_Callback(hObject, ~, ~)
possibly_create_model_settings();
gui_calib_smoother(addTab(hObject, 'Calib. smoother '));
end

function possibly_create_model_settings()
if ~isappdata(0, 'model_settings') || isempty(getappdata(0, 'model_settings'))
    gui_create_model_settings();
end
end

% Simulation!
% --------------------------------------------------------------------
function simulation_stochastic_Callback(hObject, ~, ~)
possibly_create_model_settings();
gui_stoch_simulation(addTab(hObject, 'Stoch simul. '));
end

% --------------------------------------------------------------------
function simulation_deterministic_Callback(hObject, ~, ~)
possibly_create_model_settings();
gui_determ_simulation(addTab(hObject, 'Determ simul. '));
end

function sensitivity_analysis_Callback(hObject, ~, ~)
possibly_create_model_settings();
gui_sensitivity_analysis(addTab(hObject, 'Sensitivity'));
end

% Output!
function output_shock_decomposition_Callback(hObject, ~, ~)
gui_shock_decomposition(addTab(hObject, 'Shock decomp. '));
end

% --------------------------------------------------------------------
function output_realtime_shock_decomposition_Callback(hObject, ~, ~)
gui_realtime_shock_decomposition(addTab(hObject, 'Realtime shock decomp. '));
end

% --------------------------------------------------------------------
function output_plot_shock_decomposition_Callback(hObject, ~, ~)
gui_plot_shock_decomposition(addTab(hObject, 'Plot shock decomp. '));
end

% --------------------------------------------------------------------
function output_conditional_forecast_Callback(hObject, ~)
gui_cond_forecast_preliminary(addTab(hObject, 'Prelim. Cond. forecast '), hObject);
end

% --------------------------------------------------------------------
function output_forecast_Callback(hObject, ~, ~)
gui_forecast(addTab(hObject, 'Forecast '));
end

function post_processing_epilogue_Callback(hObject, ~, ~)
gui_epilogue(addTab(hObject, 'Epilogue'));
end

% --------------------------------------------------------------------
function help_product_help_Callback(varargin)
path = [getappdata(0, 'dynare_gui_root') filesep 'resources' filesep 'html' filesep 'index.html'];
if exist(path, 'file') == 2
    open(path);
else
    disp('I looked for the Dynare GUI Help but couldn''t find it.')
    disp(['Here''s where I looked: ' path])
end
end

% --------------------------------------------------------------------
function help_dynare_manual_Callback(varargin)
global dynareroot
html_man_file = [dynareroot '..' filesep 'doc', filesep, 'dynare.html', filesep, 'index.html'];
if exist(html_man_file, 'file') == 2
    open(html_man_file);
else
    disp('I looked for the Dynare Manual but couldn''t find it.')
    disp(['Here''s where I looked: ' html_man_file])
end
end

% --------------------------------------------------------------------
function help_about_Callback(varargin)
gui_about(addTab(varargin{1}, 'About Dynare GUI'));
end

function newTab = addTab(hObject, title)
newTab = gui_tabs.add_tab(hObject, title);
end

% --- Creates and returns a handle to the GUI figure.
function h1 = dynare_gui_LayoutFcn()
appdata = [];
appdata.lastValidTag = 'figure1';
appdata.initTags = struct(...
    'handle', [], ...
    'tag', 'figure1');
h1 = figure(...
    'PaperUnits','points',...
    'Units','characters',...
    'Position',[50 30 180 40],...
    'Renderer','painters',...
    'Visible','on',...
    'Color',get(0,'defaultfigureColor'),...
    'CurrentAxesMode','manual',...
    'IntegerHandle','off',...
    'Colormap',[0 0 0.5625;0 0 0.625;0 0 0.6875;0 0 0.75;0 0 0.8125;0 0 0.875;0 0 0.9375;0 0 1;0 0.0625 1;0 0.125 1;0 0.1875 1;0 0.25 1;0 0.3125 1;0 0.375 1;0 0.4375 1;0 0.5 1;0 0.5625 1;0 0.625 1;0 0.6875 1;0 0.75 1;0 0.8125 1;0 0.875 1;0 0.9375 1;0 1 1;0.0625 1 1;0.125 1 0.9375;0.1875 1 0.875;0.25 1 0.8125;0.3125 1 0.75;0.375 1 0.6875;0.4375 1 0.625;0.5 1 0.5625;0.5625 1 0.5;0.625 1 0.4375;0.6875 1 0.375;0.75 1 0.3125;0.8125 1 0.25;0.875 1 0.1875;0.9375 1 0.125;1 1 0.0625;1 1 0;1 0.9375 0;1 0.875 0;1 0.8125 0;1 0.75 0;1 0.6875 0;1 0.625 0;1 0.5625 0;1 0.5 0;1 0.4375 0;1 0.375 0;1 0.3125 0;1 0.25 0;1 0.1875 0;1 0.125 0;1 0.0625 0;1 0 0;0.9375 0 0;0.875 0 0;0.8125 0 0;0.75 0 0;0.6875 0 0;0.625 0 0;0.5625 0 0],...
    'MenuBar','none',...
    'Name','Dynare_GUI',...
    'NumberTitle','off',...
    'Resize','on',...
    'PaperPosition',[18 180 576 432],...
    'PaperSize',[841.889736 595.275552],...
    'PaperType','A4',...
    'InvertHardcopy',get(0,'defaultfigureInvertHardcopy'),...
    'PaperOrientation','landscape',...
    'ScreenPixelsPerInchMode','manual',...
    'HandleVisibility','callback',...
    'Tag','figure1',...
    'UserData',[],...
    'CreateFcn', {@local_CreateFcn, blanks(0), appdata}, ...
    'CloseRequestFcn', @(hObject,eventdata)dynare_gui('project_exit_Callback',hObject,eventdata,guidata(hObject)));

uitoolbar(...
    'Parent',h1,...
    'Visible','off',...
    'Tag','uitoolbar1');

h3 = uimenu(...
    'Parent',h1,...
    'Accelerator','M',...
    'Label','Project',...
    'Tag','project');

uimenu(...
    'Parent',h3,...
    'Callback', @project_new_Callback, ...
    'Label','New Project ...',...
    'Tag','project_new',...
    'Accelerator','n');

appdata = [];
appdata.lastValidTag = 'project_open';

uimenu(...
    'Parent',h3,...
    'Callback', @project_open_Callback, ...
    'Label','Open Project ...',...
    'Tag','project_open',...
    'Accelerator','o');

uimenu(...
    'Parent',h3,...
    'Enable','off',...
    'Separator','on',...
    'Callback', @project_close_Callback, ...
    'Label','Close Project',...
    'Tag','project_close',...
    'Accelerator','w');

uimenu(...
    'Parent',h3,...
    'Enable','off',...
    'Callback', @project_save_Callback, ...
    'Label','Save Project',...
    'Tag','project_save',...
    'Accelerator','s');

uimenu(...
    'Parent',h3,...
    'Enable','off',...
    'Separator','on',...
    'Callback', @project_save_as_Callback, ...
    'Label','Save Project As ...',...
    'Tag','project_save_as');

uimenu(...
    'Parent',h3,...
    'Separator','on',...
    'Callback', @project_exit_Callback, ...
    'Label','Exit',...
    'Tag','project_exit');

h10 = uimenu(...
    'Parent',h1,...
    'Label','Model',...
    'Tag','model');

uimenu(...
    'Parent',h10,...
    'Enable','off',...
    'Callback', @model_load_Callback, ...
    'Label','Load main .mod/.dyn file',...
    'Tag','model_load');

uimenu(...
    'Parent',h10,...
    'Enable','off',...
    'Separator','on',...
    'Callback', @model_settings_Callback, ...
    'Label','Model settings ...',...
    'Tag','model_settings');

uimenu(...
    'Parent',h10,...
    'Enable','off',...
    'Separator','on',...
    'Callback', @model_save_snapshot_Callback, ...
    'Label','Save model snapshot',...
    'Tag','model_save_snapshot');

uimenu(...
    'Parent',h10,...
    'Enable','off',...
    'Callback', @model_load_snapshot_Callback, ...
    'Label','Load model snapshot',...
    'Tag','model_load_snapshot');

uimenu(...
    'Parent',h10,...
    'Enable','off',...
    'Callback', @model_export_Callback, ...
    'Label','Export to a .mod file',...
    'Tag','model_export');

uimenu(...
    'Parent',h10,...
    'Enable','off',...
    'Separator','on',...
    'Callback', @model_logfile_Callback, ...
    'Label','Dynare GUI log file ',...
    'Tag','model_logfile');
% 
% appdata = [];
% appdata.lastValidTag = 'Untitled_7';
% 
% uicontextmenu(...
%     'Parent',h1,...
%     'Callback',@(hObject,eventdata)dynare_gui('Untitled_7_Callback',hObject,eventdata,guidata(hObject)),...
%     'Tag','Untitled_7',...
%     'CreateFcn', {@local_CreateFcn, blanks(0), appdata} );

h18 = uimenu(...
    'Parent',h1,...
    'Label','Estimation',...
    'Tag','estimation');

uimenu(...
    'Parent',h18,...
    'Enable','off',...
    'Callback', @estimation_observed_variables_Callback, ...
    'Label','Observed variables & data file',...
    'Tag','estimation_observed_variables');

uimenu(...
    'Parent',h18,...
    'Enable','off',...
    'Callback', @estimation_parameters_shocks_Callback, ...
    'Label','Estimated parameters & shocks ...',...
    'Tag','estimation_parameters_shocks');

uimenu(...
    'Parent',h18,...
    'Enable','off',...
    'Separator','on',...
    'Callback', @estimation_run_calibrated_smoother_Callback, ...
    'Label','Run calibrated smoother',...
    'Tag','estimation_run_calibrated_smoother');

uimenu(...
    'Parent',h18,...
    'Enable','off',...
    'Callback', @estimation_run_Callback, ...
    'Label','Run estimation',...
    'Tag','estimation_run');

h23 = uimenu(...
    'Parent',h1,...
    'Label','Simulation',...
    'Tag','simulation');

uimenu(...
    'Parent',h23,...
    'Enable','off',...
    'Callback', @simulation_stochastic_Callback, ...
    'Label','Stochastic simulation',...
    'Tag','simulation_stochastic');

uimenu(...
    'Parent',h23,...
    'Enable','off',...
    'Callback', @simulation_deterministic_Callback, ...
    'Label','Deterministic simulation',...
    'Tag','simulation_deterministic');

h24 = uimenu(...
    'Parent',h1,...
    'Label','Sensitivity',...
    'Tag','sensitivity');

uimenu(...
    'Parent',h24,...
    'Enable','off',...
    'Callback', @sensitivity_analysis_Callback, ...
    'Label','Sensitivity Analysis',...
    'Tag','sensitivity_analysis');
% 
% appdata = [];
% appdata.lastValidTag = 'uipanel_welcome';

h26 = uipanel(...
    'Parent',h1,...
    'FontUnits',get(0,'defaultuipanelFontUnits'),...
    'Units','normalized',...
    'Title',blanks(0),...
    'Position',[0 0 1 1],...
    'BackgroundColor',[1 1 1],...
    'Clipping','off',...
    'Tag','uipanel_welcome',...
    'CreateFcn', {@local_CreateFcn, blanks(0), appdata} );

h_test = uicontrol(...
    'Parent',h1,...
    'Units','normalized',...
    'String','X',...
    'Style','text',...
    'FontSize',28,...
    'FontName','Tahoma',...
    'FontAngle',get(0,'defaultuicontrolFontAngle'),...
    'FontWeight','bold');

% default_char_size = get(h_test,'extent');
h_test.Visible = 'Off';

% c_width = default_char_size(3);
% c_height = default_char_size(4);

appdata = [];
appdata.lastValidTag = 'text_dyna';

% uicontrol(...
%     'Parent',h26,...
%     'FontUnits',get(0,'defaultuicontrolFontUnits'),...
%     'Units','normalized',...
%     'HorizontalAlignment',get(0,'defaultuicontrolHorizontalAlignment'),...
%     'ListboxTop',get(0,'defaultuicontrolListboxTop'),...
%     'Max',get(0,'defaultuicontrolMax'),...
%     'Min',get(0,'defaultuicontrolMin'),...
%     'SliderStep',get(0,'defaultuicontrolSliderStep'),...
%     'String','DYNARE GUI',...
%     'Style','text',...
%     'Value',get(0,'defaultuicontrolValue'),...
%     'Position',[0.5-c_width*5 0.6 c_width*10 c_height],... %'Position',[60 22.0117647058823 c_width*4 c_height],...
%     'BackgroundColor',[1 1 1],...
%     'Callback',blanks(0),...
%     'Children',[],...
%     'ForegroundColor',get(0,'defaultuicontrolForegroundColor'),...
%     'Enable',get(0,'defaultuicontrolEnable'),...
%     'TooltipString',blanks(0),...
%     'Visible',get(0,'defaultuicontrolVisible'),...
%     'KeyPressFcn',blanks(0),...
%     'KeyReleaseFcn',blanks(0),...
%     'HandleVisibility',get(0,'defaultuicontrolHandleVisibility'),...
%     'CreateFcn', {@local_CreateFcn, blanks(0), appdata} ,...
%     'DeleteFcn',blanks(0),...
%     'ButtonDownFcn',blanks(0),...
%     'Tag','text_dyna',...
%     'UserData',[],...
%     'FontSize',28,...
%     'FontName','Tahoma',...
%     'FontAngle',get(0,'defaultuicontrolFontAngle'),...
%     'FontWeight','bold');
% 
% uicontrol(...
%     'Parent',h26,...
%     'FontUnits',get(0,'defaultuicontrolFontUnits'),...
%     'Units','normalized',...
%     'HorizontalAlignment','center',...
%     'ListboxTop',get(0,'defaultuicontrolListboxTop'),...
%     'Max',get(0,'defaultuicontrolMax'),...
%     'Min',get(0,'defaultuicontrolMin'),...
%     'SliderStep',get(0,'defaultuicontrolSliderStep'),...
%     'String','Version 0.7',...
%     'Style','text',...
%     'Value',get(0,'defaultuicontrolValue'),...
%     'Position',[0,0.4,1,.1],...
%     'BackgroundColor',[1 1 1],...
%     'Callback',blanks(0),...
%     'Children',[],...
%     'ForegroundColor',get(0,'defaultuicontrolForegroundColor'),...
%     'Enable',get(0,'defaultuicontrolEnable'),...
%     'TooltipString',blanks(0),...
%     'Visible',get(0,'defaultuicontrolVisible'),...
%     'KeyPressFcn',blanks(0),...
%     'KeyReleaseFcn',blanks(0),...
%     'HandleVisibility',get(0,'defaultuicontrolHandleVisibility'),...
%     'CreateFcn', {@local_CreateFcn, blanks(0), appdata} ,...
%     'DeleteFcn',blanks(0),...
%     'ButtonDownFcn',blanks(0),...
%     'Tag','text_version',...
%     'UserData',[],...
%     'FontSize',12,...
%     'FontName',get(0,'defaultuicontrolFontName'),...
%     'FontAngle',get(0,'defaultuicontrolFontAngle'),...
%     'FontWeight',get(0,'defaultuicontrolFontWeight'));

h31 = uimenu(...
    'Parent',h1,...
    'Label','Output',...
    'Tag','output');

h39 = uimenu(...
    'Parent',h31,...
    'Enable','off', ...
    'Label','Shock Decomposition',...
    'Tag','output_shocks_dec');

uimenu(...
    'Parent',h39,...
    'Enable','off',...
    'Callback', @output_shock_decomposition_Callback, ...
    'Label','Historical',...
    'Tag','output_shocks_decomposition');

uimenu(...
    'Parent',h39,...
    'Enable','off',...
    'Callback', @output_realtime_shock_decomposition_Callback, ...
    'Label','Realtime',...
    'Tag','output_realtime_shocks_decomposition');

uimenu(...
    'Parent',h39,...
    'Enable','off',...
    'Callback', @output_plot_shock_decomposition_Callback, ...
    'Label','Plot',...
    'Tag','output_plot_shocks_decomposition');

uimenu(...
    'Parent',h31,...
    'Enable','off',...
    'Callback', @output_conditional_forecast_Callback, ...
    'Label','Conditional forecast',...
    'Tag','output_conditional_forecast');

uimenu(...
    'Parent',h31,...
    'Enable','off',...
    'Callback', @output_forecast_Callback, ...
    'Label','Forecast',...
    'Tag','output_forecast');

h33 = uimenu( ...
    'Parent', h1,...
    'Label', 'Post-processing', ...
    'Tag', 'post_processing');

uimenu( ...
    'Parent', h33, ...
    'Enable', 'off', ...
    'Callback', @post_processing_epilogue_Callback, ...
    'Label', 'Epilogue',...
    'Tag', 'post_processing_epilogue');

h34 = uimenu(...
    'Parent',h1,...
    'Separator','on',...
    'Label','Help',...
    'Tag','help');

uimenu(...
    'Parent',h34,...
    'Callback', @help_product_help_Callback, ...
    'Label','Dynare GUI help',...
    'Tag','help_product_help');

uimenu(...
    'Parent',h34,...
    'Callback', @help_dynare_manual_Callback, ...
    'Label','Dynare reference manual',...
    'Tag','help_dynare_manual');

uimenu(...
    'Parent',h34,...
    'Separator','on',...
    'Callback', @help_about_Callback, ...
    'Label','About Dynare GUI',...
    'Tag','help_about');
end

% --- Handles default GUIDE GUI creation and callback dispatch
function varargout = gui_mainfcn(gui_State, varargin)

gui_StateFields =  {'gui_Name'
                    'gui_Singleton'
                    'gui_OpeningFcn'
                    'gui_OutputFcn'
                    'gui_LayoutFcn'
                    'gui_Callback'};
gui_Mfile = '';
for i=1:length(gui_StateFields)
    if ~isfield(gui_State, gui_StateFields{i})
        error(message('dynare_gui:StateFieldNotFound', gui_StateFields{ i }, gui_Mfile));
    elseif isequal(gui_StateFields{i}, 'gui_Name')
        gui_Mfile = [gui_State.(gui_StateFields{i}), '.m'];
    end
end

numargin = length(varargin);
if numargin == 0
    % DYNARE_GUI
    % create the GUI only if we are not in the process of loading it
    % already
    gui_Create = true;
elseif local_isInvokeHGCallback(gui_State, varargin{:})
    % DYNARE_GUI('CALLBACK',hObject,eventData,handles,...)
    gui_Create = false;
else
    % DYNARE_GUI(...)
    % create the GUI and hand varargin to the openingfcn
    gui_Create = true;
end

if ~gui_Create
    % evaluate the callback now
    varargin{1} = gui_State.gui_Callback;
    if nargout
        [varargout{1:nargout}] = feval(varargin{:});
    else
        feval(varargin{:});
    end
else
    if gui_State.gui_Singleton
        gui_SingletonOpt = 'reuse';
    else
        gui_SingletonOpt = 'new';
    end

    % Check user passing 'visible' P/V pair first so that its value can be
    % used by openfig to prevent flickering
    gui_Visible = 'auto';
    gui_VisibleInput = '';
    for i=1:2:length(varargin)
        if length(varargin) == i || ~ischar(varargin{i})
            break
        end

        % Recognize 'visible' P/V pair
        len1 = min(length('visible'),length(varargin{i}));
        len2 = min(length('off'),length(varargin{i+1}));
        if ischar(varargin{i+1}) && strncmpi(varargin{i},'visible',len1) && len2 > 1
            if strncmpi(varargin{i+1},'off',len2)
                gui_Visible = 'invisible';
                gui_VisibleInput = 'off';
            elseif strncmpi(varargin{i+1},'on',len2)
                gui_Visible = 'visible';
                gui_VisibleInput = 'on';
            end
        end
    end

    % Open fig file with stored settings.  Note: This executes all component
    % specific CreateFunctions with an empty HANDLES structure.


    % Do feval on layout code in m-file if it exists
    gui_Exported = ~isempty(gui_State.gui_LayoutFcn);
    if gui_Exported
        gui_hFigure = feval(gui_State.gui_LayoutFcn);

        % make figure invisible here so that the visibility of figure is
        % consistent in OpeningFcn in the exported GUI case
        if isempty(gui_VisibleInput)
            gui_VisibleInput = gui_hFigure.Visible;
        end
        gui_hFigure.Visible = 'off';

        % openfig (called by local_openfig below) does this for guis without
        % the LayoutFcn. Be sure to do it here so guis show up on screen.
        movegui(gui_hFigure,'onscreen');
    else
        gui_hFigure = local_openfig(gui_State.gui_Name, gui_SingletonOpt, gui_Visible);
        % If the figure has InGUIInitialization it was not completely created
        % on the last pass.  Delete this handle and try again.
        if isappdata(gui_hFigure, 'InGUIInitialization')
            delete(gui_hFigure);
            gui_hFigure = local_openfig(gui_State.gui_Name, gui_SingletonOpt, gui_Visible);
        end
    end
    if isappdata(0, genvarname(['OpenGuiWhenRunning_', gui_State.gui_Name]))
        rmappdata(0,genvarname(['OpenGuiWhenRunning_', gui_State.gui_Name]));
    end

    % Set flag to indicate starting GUI initialization
    setappdata(gui_hFigure,'InGUIInitialization',1);

    % Singleton setting in the GUI M-file takes priority if different
    gui_Options.singleton = gui_State.gui_Singleton;

    if ~isappdata(gui_hFigure,'GUIOnScreen')
        % Adjust background color
        gui_hFigure.Color = get(0,'DefaultUicontrolBackgroundColor');

        % Generate HANDLES structure and store with GUIDATA. If there is
        % user set GUI data already, keep that also.
        data = guidata(gui_hFigure);
        handles = guihandles(gui_hFigure);
        if ~isempty(handles)
            if isempty(data)
                data = handles;
            else
                names = fieldnames(handles);
                for k=1:length(names)
                    data.(char(names(k)))=handles.(char(names(k)));
                end
            end
        end
        guidata(gui_hFigure, data);
    end

    % Apply input P/V pairs other than 'visible'
    for i = 1:2:length(varargin)
        if length(varargin) == i || ~ischar(varargin{i})
            break
        end
        len1 = min(length('visible'),length(varargin{i}));
        if ~strncmpi(varargin{i},'visible',len1)
            try
                gui_hFigure.(varargin{i}) = varargin{index+1};
            catch
                break
            end
        end
    end

    % If handle visibility is set to 'callback', turn it on until finished
    % with OpeningFcn
    gui_HandleVisibility = gui_hFigure.HandleVisibility;
    if strcmp(gui_HandleVisibility, 'callback')
        gui_hFigure.HandleVisibility = 'on';
    end

    feval(gui_State.gui_OpeningFcn, gui_hFigure, [], guidata(gui_hFigure), varargin{:});

    if isscalar(gui_hFigure) && ishghandle(gui_hFigure)
        % Update handle visibility
        gui_hFigure.HandleVisibility = gui_HandleVisibility;

        % Call openfig again to pick up the saved visibility or apply the
        % one passed in from the P/V pairs
        if ~gui_Exported
            gui_hFigure = local_openfig(gui_State.gui_Name, 'reuse',gui_Visible);
        elseif ~isempty(gui_VisibleInput)
            gui_hFigure.Visible = gui_VisibleInput;
        end
        if strcmpi(gui_hFigure.Visible, 'on')
            figure(gui_hFigure);
            if gui_Options.singleton
                setappdata(gui_hFigure,'GUIOnScreen', 1);
            end
        end

        % Done with GUI initialization
        if isappdata(gui_hFigure,'InGUIInitialization')
            rmappdata(gui_hFigure,'InGUIInitialization');
        end

        % If handle visibility is set to 'callback', turn it on until
        % finished with OutputFcn
        gui_HandleVisibility = gui_hFigure.HandleVisibility;
        if strcmp(gui_HandleVisibility, 'callback')
            gui_hFigure.HandleVisibility = 'on';
        end
        gui_Handles = guidata(gui_hFigure);
    else
        gui_Handles = [];
    end

    if nargout
        [varargout{1:nargout}] = feval(gui_State.gui_OutputFcn, gui_hFigure, [], gui_Handles);
    else
        feval(gui_State.gui_OutputFcn, gui_hFigure, [], gui_Handles);
    end

    if isscalar(gui_hFigure) && ishghandle(gui_hFigure)
        gui_hFigure.HandleVisibility = gui_HandleVisibility;
    end
end
end

function gui_hFigure = local_openfig(name, singleton, visible)
% openfig with three arguments was new from R13. Try to call that first, if
% failed, try the old openfig.
if nargin('openfig') == 2
    % OPENFIG did not accept 3rd input argument until R13,
    % toggle default figure visible to prevent the figure
    % from showing up too soon.
    gui_OldDefaultVisible = get(0,'defaultFigureVisible');
    set(0,'defaultFigureVisible','off');
    gui_hFigure = matlab.hg.internal.openfigLegacy(name, singleton);
    set(0,'defaultFigureVisible',gui_OldDefaultVisible);
else
    % Call version of openfig that accepts 'auto' option"
    gui_hFigure = matlab.hg.internal.openfigLegacy(name, singleton, visible);
end
end

function result = local_isInvokeHGCallback(gui_State, varargin)
try
    fhandle = functions(gui_State.gui_Callback);
    result = ~isempty(findstr(gui_State.gui_Name,fhandle.file)) || ...
             (ischar(varargin{1}) ...
              && isequal(ishghandle(varargin{2}), 1) ...
              && (~isempty(strfind(varargin{1},[get(varargin{2}, 'Tag'), '_'])) || ...
                  ~isempty(strfind(varargin{1}, '_CreateFcn'))) );
catch
    result = false;
end
end
