function gui_calib_smoother(tabId)
% function gui_calib_smoother(tabId)
% interface for the DYNARE calib_smoother command
%
% INPUTS
%   tabId:      GUI tab element which displays calib_smoother command interface
%
% OUTPUTS
%   none
%
% SPECIAL REQUIREMENTS
%   none

% Copyright (C) 2003-2019 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

global project_info dynare_gui_ options_ model_settings oo_ estim_params_ M_ bayestopt_

bg_color = char(getappdata(0,'bg_color'));
special_color = char(getappdata(0,'special_color'));

handles = [];
gui_size = gui_tools.get_gui_elements_size(tabId);

model_settings.calib_smoother = struct();

% --- PANELS -------------------------------------
handles.uipanelVars = uipanel( ...
    'Parent', tabId, ...
    'Tag', 'uipanelVars', 'BackgroundColor', special_color,...
    'Units', 'normalized', 'Position', [0.01 0.18 0.48 0.73], ...
    'Title', '', 'BorderType', 'none');

handles = gui_tabs.create_endo_vars(handles, 'calib_smoother');

handles.uipanelResults = uipanel( ...
    'Parent', tabId, ...
    'Tag', 'uipanelVars', 'BackgroundColor', bg_color,...
    'Units', 'normalized', 'Position', [0.51 0.18 0.48 0.73], ...
    'Title', 'Command options:');

uipanelResults_CreateFcn;

handles.uipanelComm = uipanel( ...
    'Parent', tabId, ...
    'Tag', 'uipanelCommOptions', ...
    'UserData', zeros(1,0),'BackgroundColor', bg_color, ...
    'Units', 'normalized', 'Position', [0.01 0.09 0.98 0.09], ...
    'Title', 'Current command options:');

% --- STATIC TEXTS -------------------------------------
uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'text7', ...
    'Style', 'text', 'BackgroundColor', bg_color,...
    'Units','normalized','Position',[0.01 0.92 0.48 0.05],...
    'FontWeight', 'bold', ...
    'String', 'Select endogenous variables that will be used in calibrated smoother:', ...
    'HorizontalAlignment', 'left');

comm_str = gui_tools.command_string('calib_smoother', model_settings.calib_smoother);

handles.calib_smoother = uicontrol( ...
    'Parent', handles.uipanelComm, ...
    'Tag', 'calib_smoother', ...
    'Style', 'text',  'BackgroundColor', bg_color,...
    'Units', 'normalized', 'Position', [0.01 0.01 0.98 0.98], ...
    'FontAngle', 'italic', ...
    'String', comm_str, ...
    'TooltipString', comm_str, ...
    'HorizontalAlignment', 'left');

% --- PUSHBUTTONS -------------------------------------
handles.pushbuttonCalib_smoother = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonSimulation', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Run calibrated smoother', ...
    'Callback', @pushbuttonCalib_smoother_Callback);

handles.pushbuttonReset = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonReset', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space*2+gui_size.button_width gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Reset', ...
    'Callback', @pushbuttonReset_Callback);

handles.pushbuttonClose = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonReset', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space*3+gui_size.button_width*2 gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Close this tab', ...
    'Callback',{@close_tab,tabId});

handles.pushbuttonResults = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonSimulation', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space*4+gui_size.button_width*3 gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Browse results...', ...
    'Enable', 'on',...
    'Callback', @pushbuttonResults_Callback);

handles.pushbuttonCloseAll = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonSimulation', ...
    'Style', 'pushbutton', ...
    'Units','normalized','Position',[gui_size.space*5+gui_size.button_width*4 gui_size.bottom gui_size.button_width gui_size.button_height],...
    'String', 'Close all output figures', ...
    'Enable', 'on',...
    'Callback', @pushbuttonCloseAll_Callback);

    function uipanelResults_CreateFcn()
        top = 1;
        dwidth = gui_size.default_width;
        dheight = gui_size.default_height;
        spc = gui_size.c_width;
        num = 1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*1.5 dheight/2],...
            'String', 'datafile:', ...
            'HorizontalAlignment', 'left');

        handles.datafile = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','edit',...
            'Units','normalized','Position',[spc*3+dwidth*1.5 top-num*dheight dwidth*1.5 dheight/2],...
            'HorizontalAlignment', 'left',...
            'String',  project_info.data_file, 'Enable', 'Off', ...
            'TooltipString','A datafile must be provided.',...
            'Callback', {@checkCommOption_Callback,'filtered_vars'});

        num = num+1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*1.5 dheight/2],...
            'String', 'filtered_vars:', ...
            'HorizontalAlignment', 'left');

        handles.filtered_vars = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units','normalized','Position',[spc*3+dwidth*1.5 top-num*dheight dwidth dheight/2],...
            'TooltipString','Triggers the computation of filtered variables.',...
            'Callback', {@checkCommOption_Callback,'filtered_vars'});

        num = num+1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*1.5 dheight/2],...
            'String', 'filter_step_ahead:', ...
            'HorizontalAlignment', 'left');

        handles.filter_step_ahead = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Style', 'edit', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*3+dwidth*1.5 top-num*dheight dwidth*1.5 dheight/2],...
            'TooltipString','Triggers the computation k-step ahead filtered values. enter in the form:[INTEGER1:INTEGER2].',...
            'HorizontalAlignment', 'left',...
            'Callback', {@checkCommOption_Callback,'filter_step_ahead'});

        num = num+1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*1.5 dheight/2],...
            'String', 'parameter set:', ...
            'HorizontalAlignment', 'left');

        handles.parameter_set = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style', 'popup', ...
            'BackgroundColor', bg_color, ...
            'Units','normalized', ...
            'Position', [spc*3+dwidth*1 top-num*dheight dwidth*2 dheight/2], ...
            'String', {'calibration','prior_mode','prior_mean','posterior_mode','posterior_mean','posterior_median','mle_mode'}, ...
            'Callback', {@checkCommOption_Callback,'parameter_set'});
        
        model_settings.calib_smoother.parameter_set = 'calibration';

        num = num+2;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*2 dheight/2],...
            'String', 'consider_all_endogenous:', ...
            'HorizontalAlignment', 'left');

        handles.select_all_vars = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units','normalized','Position',[spc*3+dwidth*2 top-num*dheight dwidth dheight/2],...
            'Callback', {@checkCommOption_Callback,'select_all_vars'});

        num = num+1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', 'BackgroundColor', bg_color,...
            'Units','normalized','Position',[spc*2 top-num*dheight dwidth*2 dheight/2],...
            'String', 'consider_only_observed:', ...
            'HorizontalAlignment', 'left');

        handles.consider_only_observed = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units','normalized','Position',[spc*3+dwidth*2 top-num*dheight dwidth dheight/2],...
            'TooltipString','Observable variables must be declared.',...
            'Callback', {@checkCommOption_Callback,'consider_only_observed'});

        function checkCommOption_Callback(hObject, ~, option_name)
            comm_options = model_settings.calib_smoother;
            value = hObject.Value;
            if strcmp(option_name, 'filter_step_ahead')
                value = hObject.String;
            end

            switch option_name
                case 'filter_step_ahead'
                    if ~isempty(value)
                        num_value = str2num(value);
                        if isempty(num_value) ...
                                || ~all(size(num_value) == [1, 2]) ...
                                || floor(num_value(1)) ~= num_value(1) || floor(num_value(2)) ~= num_value(2)
                            if isfield(comm_options,'filter_step_ahead')
                                comm_options = rmfield(comm_options,'filter_step_ahead');
                            end
                            gui_tools.show_error('Invalid input. Please define option filter_step_ahead as [INTEGER1:INTEGER2]');
                            hObject.String = '';
                        else
                            comm_options.filter_step_ahead = value;
                        end
                    else
                        if isfield(comm_options,'filter_step_ahead')
                            comm_options = rmfield(comm_options,'filter_step_ahead');
                        end
                    end
                case 'filtered_vars'
                    comm_options.filtered_vars = value;
                case 'select_all_vars'
                    if value
                        handles.consider_only_observed.Value = false;
                    end
                    handles.endoTable.Data(:, 1) = {logical(value)};
                case 'consider_only_observed'
                    if value
                        handles.select_all_vars.Value = false;
                    end
                    select_only_observed(value);
                case 'parameter_set'
                    comm_options.parameter_set = hObject.String{value};
            end
            comm_str = gui_tools.command_string('calib_smoother', comm_options);
            handles.calib_smoother.String = comm_str;
            handles.calib_smoother.TooltipString = comm_str;
            model_settings.calib_smoother =  comm_options;
        end
    end

    function pushbuttonCalib_smoother_Callback(hObject, ~)
        if isempty(options_.datafile)
            gui_tools.show_warning('Please define datafile first: go to following option of GUI menu "Estimation -> Oberved variables & data file"');
            return
        end

        estimated_model = false;
        if exist('estim_params_', 'var')
            estimated_model = true;
            save_estim_params = estim_params_;
            estim_params_ = [];
        end

        old_options = options_;
        old_oo = oo_;
        try
            backup = gui_auxiliary.backup_options(options_);
            options_ = default_option_values(M_);
            options_ = gui_auxiliary.add_backup_to_options(options_, backup);
            if ~isempty(model_settings.calib_smoother)
                names = fieldnames(model_settings.calib_smoother);
                for ii = 1: size(names,1)
                    if isempty(model_settings.calib_smoother.(names{ii}))
                        gui_auxiliary.set_command_option(names{ii}, 1, 'check_option');
                    else
                        gui_auxiliary.set_command_option(names{ii}, model_settings.calib_smoother.(names{ii}));
                    end
                end
            end

            handles.pushbuttonResults.Enable = 'off';
            gui_tools.project_log_entry('Running calibrated smoother ','...');
            [jObj, guiObj] = gui_tools.create_animated_screen('Running calibrated smoother, please wait...', tabId);
            model_settings.varlist_.calib_smoother = char(handles.endoTable.Data([handles.endoTable.Data{:,1}], 2));

            options_.plot_priors = 0;
            options_.mode_compute = 0;
            options_.graph_format = 'eps,fig';
            options_.datafile = project_info.data_file;
            options_.order = 1;
            options_.smoother = true;
            options_.order = 1;
            options_.parameter_set = handles.parameter_set.String{handles.parameter_set.Value};
            var_list_ = handles.endoTable.Data([handles.endoTable.Data{:,1}], 2);
            [oo_, M_, options_, bayestopt_] = evaluate_smoother(options_.parameter_set, var_list_, M_, oo_, options_, bayestopt_, estim_params_);

            jObj.stop;
            jObj.setBusyText('All done!');
            handles.pushbuttonResults.Enable = 'on';
            project_info.modified = true;
            project_info.mod_file_already_run = true;
            project_info.calibrated_smoother_run = true;
            gui_set_menus(true);
        catch ME
            jObj.stop;
            jObj.setBusyText('Done with errors!');
            gui_tools.show_error('Error in execution of calib_smoother command', ME, 'extended');
            uicontrol(hObject);
            options_ = old_options;
            oo_ = old_oo;
        end
        if estimated_model
            estim_params_ = save_estim_params;
        end
        delete(guiObj);
    end

    function pushbuttonReset_Callback(~, ~)
        handles.endoTable.Data(:,1) = {false};
        handles.filtered_vars.Value = 0;
        handles.filter_step_ahead.String = '';
        handles.parameter_set.Value = 1;
        handles.select_all_vars.Value =0;
        handles.consider_only_observed.Value = 0;
        model_settings.calib_smoother = struct();
        comm_str = gui_tools.command_string('calib_smoother', model_settings.calib_smoother);
        handles.calib_smoother.String = comm_str;
        handles.calib_smoother.TooltipString = comm_str;
    end

    function select_only_observed(value)
        handles.endoTable.Data(:,1) = {false};
        if value
            for ii = 1:length(options_.varobs)
                handles.endoTable.Data(strcmp(options_.varobs{ii}, handles.endoTable.Data(:,2)), 1) = {true};
            end
        end
    end

    function close_tab(~, ~, hTab)
        gui_tabs.delete_tab(hTab);
    end

    function pushbuttonResults_Callback(~, ~)
        gui_results('calib_smoother', dynare_gui_.calib_smoother_results);
    end

    function pushbuttonCloseAll_Callback(~, ~)
        gui_tools.close_all_figures();
    end
end
